
// Copyright (c) 2014 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef CSSPropertyNames_h
#define CSSPropertyNames_h

#include "core/css/parser/CSSParserMode.h"
#include "wtf/HashFunctions.h"
#include "wtf/HashTraits.h"
#include <string.h>

namespace WTF {
class AtomicString;
class String;
}

namespace blink {

enum CSSPropertyID {
    CSSPropertyInvalid = 0,
    CSSPropertyVariable = 1,
    CSSPropertyColor = 2,
    CSSPropertyDirection = 3,
    CSSPropertyFontFamily = 4,
    CSSPropertyFontKerning = 5,
    CSSPropertyFontSize = 6,
    CSSPropertyFontSizeAdjust = 7,
    CSSPropertyFontStretch = 8,
    CSSPropertyFontStyle = 9,
    CSSPropertyFontVariant = 10,
    CSSPropertyFontVariantLigatures = 11,
    CSSPropertyFontWeight = 12,
    CSSPropertyFontFeatureSettings = 13,
    CSSPropertyWebkitFontSmoothing = 14,
    CSSPropertyWebkitLocale = 15,
    CSSPropertyTextOrientation = 16,
    CSSPropertyWebkitTextOrientation = 17,
    CSSPropertyWritingMode = 18,
    CSSPropertyWebkitWritingMode = 19,
    CSSPropertyTextRendering = 20,
    CSSPropertyZoom = 21,
    CSSPropertyAlignContent = 22,
    CSSPropertyAlignItems = 23,
    CSSPropertyAlignmentBaseline = 24,
    CSSPropertyAlignSelf = 25,
    CSSPropertyAnimationDelay = 26,
    CSSPropertyAnimationDirection = 27,
    CSSPropertyAnimationDuration = 28,
    CSSPropertyAnimationFillMode = 29,
    CSSPropertyAnimationIterationCount = 30,
    CSSPropertyAnimationName = 31,
    CSSPropertyAnimationPlayState = 32,
    CSSPropertyAnimationTimingFunction = 33,
    CSSPropertyBackdropFilter = 34,
    CSSPropertyBackfaceVisibility = 35,
    CSSPropertyBackgroundAttachment = 36,
    CSSPropertyBackgroundBlendMode = 37,
    CSSPropertyBackgroundClip = 38,
    CSSPropertyBackgroundColor = 39,
    CSSPropertyBackgroundImage = 40,
    CSSPropertyBackgroundOrigin = 41,
    CSSPropertyBackgroundPositionX = 42,
    CSSPropertyBackgroundPositionY = 43,
    CSSPropertyBackgroundRepeatX = 44,
    CSSPropertyBackgroundRepeatY = 45,
    CSSPropertyBackgroundSize = 46,
    CSSPropertyBaselineShift = 47,
    CSSPropertyBorderBottomColor = 48,
    CSSPropertyBorderBottomLeftRadius = 49,
    CSSPropertyBorderBottomRightRadius = 50,
    CSSPropertyBorderBottomStyle = 51,
    CSSPropertyBorderBottomWidth = 52,
    CSSPropertyBorderCollapse = 53,
    CSSPropertyBorderImageOutset = 54,
    CSSPropertyBorderImageRepeat = 55,
    CSSPropertyBorderImageSlice = 56,
    CSSPropertyBorderImageSource = 57,
    CSSPropertyBorderImageWidth = 58,
    CSSPropertyBorderLeftColor = 59,
    CSSPropertyBorderLeftStyle = 60,
    CSSPropertyBorderLeftWidth = 61,
    CSSPropertyBorderRightColor = 62,
    CSSPropertyBorderRightStyle = 63,
    CSSPropertyBorderRightWidth = 64,
    CSSPropertyBorderTopColor = 65,
    CSSPropertyBorderTopLeftRadius = 66,
    CSSPropertyBorderTopRightRadius = 67,
    CSSPropertyBorderTopStyle = 68,
    CSSPropertyBorderTopWidth = 69,
    CSSPropertyBottom = 70,
    CSSPropertyBoxShadow = 71,
    CSSPropertyBoxSizing = 72,
    CSSPropertyBufferedRendering = 73,
    CSSPropertyCaptionSide = 74,
    CSSPropertyClear = 75,
    CSSPropertyClip = 76,
    CSSPropertyClipPath = 77,
    CSSPropertyClipRule = 78,
    CSSPropertyColorInterpolation = 79,
    CSSPropertyColorInterpolationFilters = 80,
    CSSPropertyColorRendering = 81,
    CSSPropertyColumnFill = 82,
    CSSPropertyContain = 83,
    CSSPropertyContent = 84,
    CSSPropertyCounterIncrement = 85,
    CSSPropertyCounterReset = 86,
    CSSPropertyCursor = 87,
    CSSPropertyCx = 88,
    CSSPropertyCy = 89,
    CSSPropertyD = 90,
    CSSPropertyDisplay = 91,
    CSSPropertyDominantBaseline = 92,
    CSSPropertyEmptyCells = 93,
    CSSPropertyFill = 94,
    CSSPropertyFillOpacity = 95,
    CSSPropertyFillRule = 96,
    CSSPropertyFilter = 97,
    CSSPropertyFlexBasis = 98,
    CSSPropertyFlexDirection = 99,
    CSSPropertyFlexGrow = 100,
    CSSPropertyFlexShrink = 101,
    CSSPropertyFlexWrap = 102,
    CSSPropertyFloat = 103,
    CSSPropertyFloodColor = 104,
    CSSPropertyFloodOpacity = 105,
    CSSPropertyGridAutoColumns = 106,
    CSSPropertyGridAutoFlow = 107,
    CSSPropertyGridAutoRows = 108,
    CSSPropertyGridColumnEnd = 109,
    CSSPropertyGridColumnGap = 110,
    CSSPropertyGridColumnStart = 111,
    CSSPropertyGridRowEnd = 112,
    CSSPropertyGridRowGap = 113,
    CSSPropertyGridRowStart = 114,
    CSSPropertyGridTemplateAreas = 115,
    CSSPropertyGridTemplateColumns = 116,
    CSSPropertyGridTemplateRows = 117,
    CSSPropertyHeight = 118,
    CSSPropertyImageRendering = 119,
    CSSPropertyImageOrientation = 120,
    CSSPropertyIsolation = 121,
    CSSPropertyJustifyContent = 122,
    CSSPropertyJustifyItems = 123,
    CSSPropertyJustifySelf = 124,
    CSSPropertyLeft = 125,
    CSSPropertyLetterSpacing = 126,
    CSSPropertyLightingColor = 127,
    CSSPropertyLineHeight = 128,
    CSSPropertyListStyleImage = 129,
    CSSPropertyListStylePosition = 130,
    CSSPropertyListStyleType = 131,
    CSSPropertyMarginBottom = 132,
    CSSPropertyMarginLeft = 133,
    CSSPropertyMarginRight = 134,
    CSSPropertyMarginTop = 135,
    CSSPropertyMarkerEnd = 136,
    CSSPropertyMarkerMid = 137,
    CSSPropertyMarkerStart = 138,
    CSSPropertyMask = 139,
    CSSPropertyMaskSourceType = 140,
    CSSPropertyMaskType = 141,
    CSSPropertyMaxHeight = 142,
    CSSPropertyMaxWidth = 143,
    CSSPropertyMinHeight = 144,
    CSSPropertyMinWidth = 145,
    CSSPropertyMixBlendMode = 146,
    CSSPropertyMotionOffset = 147,
    CSSPropertyMotionPath = 148,
    CSSPropertyMotionRotation = 149,
    CSSPropertyObjectFit = 150,
    CSSPropertyObjectPosition = 151,
    CSSPropertyOpacity = 152,
    CSSPropertyOrder = 153,
    CSSPropertyOrphans = 154,
    CSSPropertyOutlineColor = 155,
    CSSPropertyOutlineOffset = 156,
    CSSPropertyOutlineStyle = 157,
    CSSPropertyOutlineWidth = 158,
    CSSPropertyOverflowWrap = 159,
    CSSPropertyOverflowX = 160,
    CSSPropertyOverflowY = 161,
    CSSPropertyPaddingBottom = 162,
    CSSPropertyPaddingLeft = 163,
    CSSPropertyPaddingRight = 164,
    CSSPropertyPaddingTop = 165,
    CSSPropertyPageBreakAfter = 166,
    CSSPropertyPageBreakBefore = 167,
    CSSPropertyPageBreakInside = 168,
    CSSPropertyPaintOrder = 169,
    CSSPropertyPerspective = 170,
    CSSPropertyPerspectiveOrigin = 171,
    CSSPropertyPointerEvents = 172,
    CSSPropertyPosition = 173,
    CSSPropertyQuotes = 174,
    CSSPropertyResize = 175,
    CSSPropertyRight = 176,
    CSSPropertyR = 177,
    CSSPropertyRx = 178,
    CSSPropertyRy = 179,
    CSSPropertyScrollBehavior = 180,
    CSSPropertyScrollSnapType = 181,
    CSSPropertyScrollSnapPointsX = 182,
    CSSPropertyScrollSnapPointsY = 183,
    CSSPropertyScrollSnapDestination = 184,
    CSSPropertyScrollSnapCoordinate = 185,
    CSSPropertyShapeImageThreshold = 186,
    CSSPropertyShapeMargin = 187,
    CSSPropertyShapeOutside = 188,
    CSSPropertyShapeRendering = 189,
    CSSPropertySize = 190,
    CSSPropertySpeak = 191,
    CSSPropertyStopColor = 192,
    CSSPropertyStopOpacity = 193,
    CSSPropertyStroke = 194,
    CSSPropertyStrokeDasharray = 195,
    CSSPropertyStrokeDashoffset = 196,
    CSSPropertyStrokeLinecap = 197,
    CSSPropertyStrokeLinejoin = 198,
    CSSPropertyStrokeMiterlimit = 199,
    CSSPropertyStrokeOpacity = 200,
    CSSPropertyStrokeWidth = 201,
    CSSPropertyTableLayout = 202,
    CSSPropertyTabSize = 203,
    CSSPropertyTextAlign = 204,
    CSSPropertyTextAlignLast = 205,
    CSSPropertyTextAnchor = 206,
    CSSPropertyTextCombineUpright = 207,
    CSSPropertyTextDecoration = 208,
    CSSPropertyTextDecorationColor = 209,
    CSSPropertyTextDecorationLine = 210,
    CSSPropertyTextDecorationStyle = 211,
    CSSPropertyTextIndent = 212,
    CSSPropertyTextJustify = 213,
    CSSPropertyTextOverflow = 214,
    CSSPropertyTextShadow = 215,
    CSSPropertyTextTransform = 216,
    CSSPropertyTextUnderlinePosition = 217,
    CSSPropertyTop = 218,
    CSSPropertyTouchAction = 219,
    CSSPropertyTransform = 220,
    CSSPropertyTransformOrigin = 221,
    CSSPropertyTransformStyle = 222,
    CSSPropertyTranslate = 223,
    CSSPropertyRotate = 224,
    CSSPropertyScale = 225,
    CSSPropertyTransitionDelay = 226,
    CSSPropertyTransitionDuration = 227,
    CSSPropertyTransitionProperty = 228,
    CSSPropertyTransitionTimingFunction = 229,
    CSSPropertyUnicodeBidi = 230,
    CSSPropertyVectorEffect = 231,
    CSSPropertyVerticalAlign = 232,
    CSSPropertyVisibility = 233,
    CSSPropertyX = 234,
    CSSPropertyY = 235,
    CSSPropertyWebkitAppearance = 236,
    CSSPropertyWebkitAppRegion = 237,
    CSSPropertyWebkitBackgroundClip = 238,
    CSSPropertyWebkitBackgroundComposite = 239,
    CSSPropertyWebkitBackgroundOrigin = 240,
    CSSPropertyWebkitBorderHorizontalSpacing = 241,
    CSSPropertyWebkitBorderImage = 242,
    CSSPropertyWebkitBorderVerticalSpacing = 243,
    CSSPropertyWebkitBoxAlign = 244,
    CSSPropertyWebkitBoxDecorationBreak = 245,
    CSSPropertyWebkitBoxDirection = 246,
    CSSPropertyWebkitBoxFlex = 247,
    CSSPropertyWebkitBoxFlexGroup = 248,
    CSSPropertyWebkitBoxLines = 249,
    CSSPropertyWebkitBoxOrdinalGroup = 250,
    CSSPropertyWebkitBoxOrient = 251,
    CSSPropertyWebkitBoxPack = 252,
    CSSPropertyWebkitBoxReflect = 253,
    CSSPropertyWebkitClipPath = 254,
    CSSPropertyWebkitColumnBreakAfter = 255,
    CSSPropertyWebkitColumnBreakBefore = 256,
    CSSPropertyWebkitColumnBreakInside = 257,
    CSSPropertyWebkitColumnCount = 258,
    CSSPropertyWebkitColumnGap = 259,
    CSSPropertyWebkitColumnRuleColor = 260,
    CSSPropertyWebkitColumnRuleStyle = 261,
    CSSPropertyWebkitColumnRuleWidth = 262,
    CSSPropertyWebkitColumnSpan = 263,
    CSSPropertyWebkitColumnWidth = 264,
    CSSPropertyWebkitFilter = 265,
    CSSPropertyWebkitHighlight = 266,
    CSSPropertyWebkitHyphenateCharacter = 267,
    CSSPropertyWebkitLineBreak = 268,
    CSSPropertyWebkitLineClamp = 269,
    CSSPropertyWebkitMarginAfterCollapse = 270,
    CSSPropertyWebkitMarginBeforeCollapse = 271,
    CSSPropertyWebkitMarginBottomCollapse = 272,
    CSSPropertyWebkitMarginTopCollapse = 273,
    CSSPropertyWebkitMaskBoxImageOutset = 274,
    CSSPropertyWebkitMaskBoxImageRepeat = 275,
    CSSPropertyWebkitMaskBoxImageSlice = 276,
    CSSPropertyWebkitMaskBoxImageSource = 277,
    CSSPropertyWebkitMaskBoxImageWidth = 278,
    CSSPropertyWebkitMaskClip = 279,
    CSSPropertyWebkitMaskComposite = 280,
    CSSPropertyWebkitMaskImage = 281,
    CSSPropertyWebkitMaskOrigin = 282,
    CSSPropertyWebkitMaskPositionX = 283,
    CSSPropertyWebkitMaskPositionY = 284,
    CSSPropertyWebkitMaskRepeatX = 285,
    CSSPropertyWebkitMaskRepeatY = 286,
    CSSPropertyWebkitMaskSize = 287,
    CSSPropertyWebkitPerspectiveOriginX = 288,
    CSSPropertyWebkitPerspectiveOriginY = 289,
    CSSPropertyWebkitPrintColorAdjust = 290,
    CSSPropertyWebkitRtlOrdering = 291,
    CSSPropertyWebkitRubyPosition = 292,
    CSSPropertyWebkitTapHighlightColor = 293,
    CSSPropertyWebkitTextCombine = 294,
    CSSPropertyWebkitTextEmphasisColor = 295,
    CSSPropertyWebkitTextEmphasisPosition = 296,
    CSSPropertyWebkitTextEmphasisStyle = 297,
    CSSPropertyWebkitTextFillColor = 298,
    CSSPropertyWebkitTextSecurity = 299,
    CSSPropertyWebkitTextStrokeColor = 300,
    CSSPropertyWebkitTextStrokeWidth = 301,
    CSSPropertyWebkitTransformOriginX = 302,
    CSSPropertyWebkitTransformOriginY = 303,
    CSSPropertyWebkitTransformOriginZ = 304,
    CSSPropertyWebkitUserDrag = 305,
    CSSPropertyWebkitUserModify = 306,
    CSSPropertyWebkitUserSelect = 307,
    CSSPropertyWhiteSpace = 308,
    CSSPropertyWidows = 309,
    CSSPropertyWidth = 310,
    CSSPropertyWillChange = 311,
    CSSPropertyWordBreak = 312,
    CSSPropertyWordSpacing = 313,
    CSSPropertyWordWrap = 314,
    CSSPropertyZIndex = 315,
    CSSPropertyWebkitBorderEndColor = 316,
    CSSPropertyWebkitBorderEndStyle = 317,
    CSSPropertyWebkitBorderEndWidth = 318,
    CSSPropertyWebkitBorderStartColor = 319,
    CSSPropertyWebkitBorderStartStyle = 320,
    CSSPropertyWebkitBorderStartWidth = 321,
    CSSPropertyWebkitBorderBeforeColor = 322,
    CSSPropertyWebkitBorderBeforeStyle = 323,
    CSSPropertyWebkitBorderBeforeWidth = 324,
    CSSPropertyWebkitBorderAfterColor = 325,
    CSSPropertyWebkitBorderAfterStyle = 326,
    CSSPropertyWebkitBorderAfterWidth = 327,
    CSSPropertyWebkitMarginEnd = 328,
    CSSPropertyWebkitMarginStart = 329,
    CSSPropertyWebkitMarginBefore = 330,
    CSSPropertyWebkitMarginAfter = 331,
    CSSPropertyWebkitPaddingEnd = 332,
    CSSPropertyWebkitPaddingStart = 333,
    CSSPropertyWebkitPaddingBefore = 334,
    CSSPropertyWebkitPaddingAfter = 335,
    CSSPropertyWebkitLogicalWidth = 336,
    CSSPropertyWebkitLogicalHeight = 337,
    CSSPropertyWebkitMinLogicalWidth = 338,
    CSSPropertyWebkitMinLogicalHeight = 339,
    CSSPropertyWebkitMaxLogicalWidth = 340,
    CSSPropertyWebkitMaxLogicalHeight = 341,
    CSSPropertyAll = 342,
    CSSPropertyFontDisplay = 343,
    CSSPropertyMaxZoom = 344,
    CSSPropertyMinZoom = 345,
    CSSPropertyOrientation = 346,
    CSSPropertyPage = 347,
    CSSPropertySrc = 348,
    CSSPropertyUnicodeRange = 349,
    CSSPropertyUserZoom = 350,
    CSSPropertyWebkitFontSizeDelta = 351,
    CSSPropertyWebkitTextDecorationsInEffect = 352,
    CSSPropertyAnimation = 353,
    CSSPropertyBackground = 354,
    CSSPropertyBackgroundPosition = 355,
    CSSPropertyBackgroundRepeat = 356,
    CSSPropertyBorder = 357,
    CSSPropertyBorderBottom = 358,
    CSSPropertyBorderColor = 359,
    CSSPropertyBorderImage = 360,
    CSSPropertyBorderLeft = 361,
    CSSPropertyBorderRadius = 362,
    CSSPropertyBorderRight = 363,
    CSSPropertyBorderSpacing = 364,
    CSSPropertyBorderStyle = 365,
    CSSPropertyBorderTop = 366,
    CSSPropertyBorderWidth = 367,
    CSSPropertyFlex = 368,
    CSSPropertyFlexFlow = 369,
    CSSPropertyFont = 370,
    CSSPropertyGrid = 371,
    CSSPropertyGridArea = 372,
    CSSPropertyGridColumn = 373,
    CSSPropertyGridGap = 374,
    CSSPropertyGridRow = 375,
    CSSPropertyGridTemplate = 376,
    CSSPropertyListStyle = 377,
    CSSPropertyMargin = 378,
    CSSPropertyMarker = 379,
    CSSPropertyMotion = 380,
    CSSPropertyOutline = 381,
    CSSPropertyOverflow = 382,
    CSSPropertyPadding = 383,
    CSSPropertyTransition = 384,
    CSSPropertyWebkitBorderAfter = 385,
    CSSPropertyWebkitBorderBefore = 386,
    CSSPropertyWebkitBorderEnd = 387,
    CSSPropertyWebkitBorderStart = 388,
    CSSPropertyWebkitColumnRule = 389,
    CSSPropertyWebkitColumns = 390,
    CSSPropertyWebkitMarginCollapse = 391,
    CSSPropertyWebkitMask = 392,
    CSSPropertyWebkitMaskBoxImage = 393,
    CSSPropertyWebkitMaskPosition = 394,
    CSSPropertyWebkitMaskRepeat = 395,
    CSSPropertyWebkitTextEmphasis = 396,
    CSSPropertyWebkitTextStroke = 397,
    CSSPropertyAliasEpubCaptionSide = 586,
    CSSPropertyAliasEpubTextCombine = 806,
    CSSPropertyAliasEpubTextEmphasis = 908,
    CSSPropertyAliasEpubTextEmphasisColor = 807,
    CSSPropertyAliasEpubTextEmphasisStyle = 809,
    CSSPropertyAliasEpubTextOrientation = 529,
    CSSPropertyAliasEpubTextTransform = 728,
    CSSPropertyAliasEpubWordBreak = 824,
    CSSPropertyAliasEpubWritingMode = 531,
    CSSPropertyAliasWebkitAlignContent = 534,
    CSSPropertyAliasWebkitAlignItems = 535,
    CSSPropertyAliasWebkitAlignSelf = 537,
    CSSPropertyAliasWebkitAnimation = 865,
    CSSPropertyAliasWebkitAnimationDelay = 538,
    CSSPropertyAliasWebkitAnimationDirection = 539,
    CSSPropertyAliasWebkitAnimationDuration = 540,
    CSSPropertyAliasWebkitAnimationFillMode = 541,
    CSSPropertyAliasWebkitAnimationIterationCount = 542,
    CSSPropertyAliasWebkitAnimationName = 543,
    CSSPropertyAliasWebkitAnimationPlayState = 544,
    CSSPropertyAliasWebkitAnimationTimingFunction = 545,
    CSSPropertyAliasWebkitBackfaceVisibility = 547,
    CSSPropertyAliasWebkitBackgroundSize = 558,
    CSSPropertyAliasWebkitBorderBottomLeftRadius = 561,
    CSSPropertyAliasWebkitBorderBottomRightRadius = 562,
    CSSPropertyAliasWebkitBorderRadius = 874,
    CSSPropertyAliasWebkitBorderTopLeftRadius = 578,
    CSSPropertyAliasWebkitBorderTopRightRadius = 579,
    CSSPropertyAliasWebkitBoxShadow = 583,
    CSSPropertyAliasWebkitBoxSizing = 584,
    CSSPropertyAliasWebkitFlex = 880,
    CSSPropertyAliasWebkitFlexBasis = 610,
    CSSPropertyAliasWebkitFlexDirection = 611,
    CSSPropertyAliasWebkitFlexFlow = 881,
    CSSPropertyAliasWebkitFlexGrow = 612,
    CSSPropertyAliasWebkitFlexShrink = 613,
    CSSPropertyAliasWebkitFlexWrap = 614,
    CSSPropertyAliasWebkitFontFeatureSettings = 525,
    CSSPropertyAliasWebkitJustifyContent = 634,
    CSSPropertyAliasWebkitOpacity = 664,
    CSSPropertyAliasWebkitOrder = 665,
    CSSPropertyAliasWebkitPerspective = 682,
    CSSPropertyAliasWebkitPerspectiveOrigin = 683,
    CSSPropertyAliasWebkitShapeImageThreshold = 698,
    CSSPropertyAliasWebkitShapeMargin = 699,
    CSSPropertyAliasWebkitShapeOutside = 700,
    CSSPropertyAliasWebkitTransform = 732,
    CSSPropertyAliasWebkitTransformOrigin = 733,
    CSSPropertyAliasWebkitTransformStyle = 734,
    CSSPropertyAliasWebkitTransition = 896,
    CSSPropertyAliasWebkitTransitionDelay = 738,
    CSSPropertyAliasWebkitTransitionDuration = 739,
    CSSPropertyAliasWebkitTransitionProperty = 740,
    CSSPropertyAliasWebkitTransitionTimingFunction = 741,
};

const int firstCSSProperty = 2;
const int numCSSProperties = 396;
const int lastCSSProperty = 397;
const int lastUnresolvedCSSProperty = 908;
const size_t maxCSSPropertyNameLength = 40;

const char* getPropertyName(CSSPropertyID);
const WTF::AtomicString& getPropertyNameAtomicString(CSSPropertyID);
WTF::String getPropertyNameString(CSSPropertyID);
WTF::String getJSPropertyName(CSSPropertyID);

inline CSSPropertyID convertToCSSPropertyID(int value)
{
    ASSERT((value >= firstCSSProperty && value <= lastCSSProperty) || value == CSSPropertyInvalid || value == CSSPropertyVariable);
    return static_cast<CSSPropertyID>(value);
}

inline CSSPropertyID resolveCSSPropertyID(CSSPropertyID id)
{
    return convertToCSSPropertyID(id & ~512);
}

inline bool isPropertyAlias(CSSPropertyID id) { return id & 512; }

CSSPropertyID unresolvedCSSPropertyID(const WTF::String&);

CSSPropertyID cssPropertyID(const WTF::String&);

} // namespace blink

namespace WTF {
template<> struct DefaultHash<blink::CSSPropertyID> { typedef IntHash<unsigned> Hash; };
template<> struct HashTraits<blink::CSSPropertyID> : GenericHashTraits<blink::CSSPropertyID> {
    static const bool emptyValueIsZero = true;
    static void constructDeletedValue(blink::CSSPropertyID& slot, bool) { slot = static_cast<blink::CSSPropertyID>(blink::lastUnresolvedCSSProperty + 1); }
    static bool isDeletedValue(blink::CSSPropertyID value) { return value == (blink::lastUnresolvedCSSProperty + 1); }
};
}

#endif // CSSPropertyNames_h
