// Copyright 2013 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef CONTENT_COMMON_BACKGROUND_SYNC_SERVICE_MOJOM_H_
#define CONTENT_COMMON_BACKGROUND_SYNC_SERVICE_MOJOM_H_

#include <stdint.h>

#include "mojo/public/cpp/bindings/array.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr_info.h"
#include "mojo/public/cpp/bindings/associated_interface_request.h"
#include "mojo/public/cpp/bindings/callback.h"
#include "mojo/public/cpp/bindings/interface_ptr.h"
#include "mojo/public/cpp/bindings/interface_request.h"
#include "mojo/public/cpp/bindings/lib/control_message_handler.h"
#include "mojo/public/cpp/bindings/lib/control_message_proxy.h"
#include "mojo/public/cpp/bindings/map.h"
#include "mojo/public/cpp/bindings/message_filter.h"
#include "mojo/public/cpp/bindings/no_interface.h"
#include "mojo/public/cpp/bindings/string.h"
#include "mojo/public/cpp/bindings/struct_ptr.h"
#include "mojo/public/cpp/bindings/struct_traits.h"
#include "content/common/background_sync_service.mojom-internal.h"
#include "content/public/common/background_sync.mojom.h"
#include "content/public/common/permission_status.mojom.h"
#include "content/public/common/service_worker_event_status.mojom.h"
namespace content {

  
enum BackgroundSyncError : int32_t {
  BACKGROUND_SYNC_ERROR_NONE,
  BACKGROUND_SYNC_ERROR_STORAGE,
  BACKGROUND_SYNC_ERROR_NOT_FOUND,
  BACKGROUND_SYNC_ERROR_NO_SERVICE_WORKER,
  BACKGROUND_SYNC_ERROR_NOT_ALLOWED,
  BACKGROUND_SYNC_ERROR_MAX = BACKGROUND_SYNC_ERROR_NOT_ALLOWED,
};

bool BackgroundSyncError_IsValidValue(BackgroundSyncError value);
  
enum BackgroundSyncState : int32_t {
  BACKGROUND_SYNC_STATE_PENDING,
  BACKGROUND_SYNC_STATE_FIRING,
  BACKGROUND_SYNC_STATE_UNREGISTERED_WHILE_FIRING,
  BACKGROUND_SYNC_STATE_REREGISTERED_WHILE_FIRING,
  BACKGROUND_SYNC_STATE_FAILED,
  BACKGROUND_SYNC_STATE_SUCCESS,
  BACKGROUND_SYNC_STATE_UNREGISTERED,
};

bool BackgroundSyncState_IsValidValue(BackgroundSyncState value);
  
enum BackgroundSyncEventLastChance : int32_t {
  BACKGROUND_SYNC_EVENT_LAST_CHANCE_IS_NOT_LAST_CHANCE,
  BACKGROUND_SYNC_EVENT_LAST_CHANCE_IS_LAST_CHANCE,
};

bool BackgroundSyncEventLastChance_IsValidValue(BackgroundSyncEventLastChance value);
class BackgroundSyncService;
using BackgroundSyncServicePtr = mojo::InterfacePtr<BackgroundSyncService>;
using BackgroundSyncServiceAssociatedPtr = mojo::AssociatedInterfacePtr<BackgroundSyncService>;

class BackgroundSyncServiceClient;
using BackgroundSyncServiceClientPtr = mojo::InterfacePtr<BackgroundSyncServiceClient>;
using BackgroundSyncServiceClientAssociatedPtr = mojo::AssociatedInterfacePtr<BackgroundSyncServiceClient>;


class BackgroundSyncServiceProxy;
class BackgroundSyncServiceStub;

class BackgroundSyncServiceRequestValidator;
class BackgroundSyncServiceResponseValidator;

class BackgroundSyncService {
 public:
  static const char Name_[];
  static const uint32_t Version_ = 0;
  static const bool PassesAssociatedKinds_ = false;

  using GenericInterface = ::content::BackgroundSyncService;

  using Proxy_ = BackgroundSyncServiceProxy;
  using Stub_ = BackgroundSyncServiceStub;

  using RequestValidator_ = BackgroundSyncServiceRequestValidator;
  using ResponseValidator_ = BackgroundSyncServiceResponseValidator;

  virtual ~BackgroundSyncService() {}

  using RegisterCallback = mojo::Callback<void(BackgroundSyncError, content::SyncRegistrationPtr)>;
  virtual void Register(content::SyncRegistrationPtr options, int64_t service_worker_registration_id, bool requested_from_service_worker, const RegisterCallback& callback) = 0;

  using GetRegistrationCallback = mojo::Callback<void(BackgroundSyncError, content::SyncRegistrationPtr)>;
  virtual void GetRegistration(content::BackgroundSyncPeriodicity periodicity, const mojo::String& tag, int64_t service_worker_registration_id, const GetRegistrationCallback& callback) = 0;

  using GetRegistrationsCallback = mojo::Callback<void(BackgroundSyncError, mojo::Array<content::SyncRegistrationPtr>)>;
  virtual void GetRegistrations(content::BackgroundSyncPeriodicity periodicity, int64_t service_worker_registration_id, const GetRegistrationsCallback& callback) = 0;

  using UnregisterCallback = mojo::Callback<void(BackgroundSyncError)>;
  virtual void Unregister(int64_t handle_id, int64_t service_worker_registration_id, const UnregisterCallback& callback) = 0;

  using GetPermissionStatusCallback = mojo::Callback<void(BackgroundSyncError, content::PermissionStatus)>;
  virtual void GetPermissionStatus(content::BackgroundSyncPeriodicity periodicity, int64_t service_worker_registration_id, const GetPermissionStatusCallback& callback) = 0;

  using DuplicateRegistrationHandleCallback = mojo::Callback<void(BackgroundSyncError, content::SyncRegistrationPtr)>;
  virtual void DuplicateRegistrationHandle(int64_t handle_id, const DuplicateRegistrationHandleCallback& callback) = 0;

  virtual void ReleaseRegistration(int64_t handle_id) = 0;

  using NotifyWhenFinishedCallback = mojo::Callback<void(BackgroundSyncError, BackgroundSyncState)>;
  virtual void NotifyWhenFinished(int64_t handle_id, const NotifyWhenFinishedCallback& callback) = 0;
};

class BackgroundSyncServiceClientProxy;
class BackgroundSyncServiceClientStub;

class BackgroundSyncServiceClientRequestValidator;
class BackgroundSyncServiceClientResponseValidator;

class BackgroundSyncServiceClient {
 public:
  static const char Name_[];
  static const uint32_t Version_ = 0;
  static const bool PassesAssociatedKinds_ = false;

  using GenericInterface = ::content::BackgroundSyncServiceClient;

  using Proxy_ = BackgroundSyncServiceClientProxy;
  using Stub_ = BackgroundSyncServiceClientStub;

  using RequestValidator_ = BackgroundSyncServiceClientRequestValidator;
  using ResponseValidator_ = BackgroundSyncServiceClientResponseValidator;

  virtual ~BackgroundSyncServiceClient() {}

  using SyncCallback = mojo::Callback<void(content::ServiceWorkerEventStatus)>;
  virtual void Sync(int64_t handle_id, BackgroundSyncEventLastChance last_chance, const SyncCallback& callback) = 0;
};

class BackgroundSyncServiceProxy
    : public BackgroundSyncService,
      public mojo::internal::ControlMessageProxy {
 public:
  explicit BackgroundSyncServiceProxy(mojo::MessageReceiverWithResponder* receiver);
  void Register(
      content::SyncRegistrationPtr options, int64_t service_worker_registration_id, bool requested_from_service_worker, const RegisterCallback& callback
  ) override;
  void GetRegistration(
      content::BackgroundSyncPeriodicity periodicity, const mojo::String& tag, int64_t service_worker_registration_id, const GetRegistrationCallback& callback
  ) override;
  void GetRegistrations(
      content::BackgroundSyncPeriodicity periodicity, int64_t service_worker_registration_id, const GetRegistrationsCallback& callback
  ) override;
  void Unregister(
      int64_t handle_id, int64_t service_worker_registration_id, const UnregisterCallback& callback
  ) override;
  void GetPermissionStatus(
      content::BackgroundSyncPeriodicity periodicity, int64_t service_worker_registration_id, const GetPermissionStatusCallback& callback
  ) override;
  void DuplicateRegistrationHandle(
      int64_t handle_id, const DuplicateRegistrationHandleCallback& callback
  ) override;
  void ReleaseRegistration(
      int64_t handle_id
  ) override;
  void NotifyWhenFinished(
      int64_t handle_id, const NotifyWhenFinishedCallback& callback
  ) override;

  mojo::internal::SerializationContext* serialization_context() {
    return &serialization_context_;
  }

 private:
  mojo::internal::SerializationContext serialization_context_;
};

class BackgroundSyncServiceClientProxy
    : public BackgroundSyncServiceClient,
      public mojo::internal::ControlMessageProxy {
 public:
  explicit BackgroundSyncServiceClientProxy(mojo::MessageReceiverWithResponder* receiver);
  void Sync(
      int64_t handle_id, BackgroundSyncEventLastChance last_chance, const SyncCallback& callback
  ) override;

  mojo::internal::SerializationContext* serialization_context() {
    return &serialization_context_;
  }

 private:
  mojo::internal::SerializationContext serialization_context_;
};
class BackgroundSyncServiceStub : public mojo::MessageReceiverWithResponderStatus {
 public:
  BackgroundSyncServiceStub();
  ~BackgroundSyncServiceStub() override;
  void set_sink(BackgroundSyncService* sink) { sink_ = sink; }
  BackgroundSyncService* sink() { return sink_; }
  mojo::internal::SerializationContext* serialization_context() {
    return &serialization_context_;
  }

  bool Accept(mojo::Message* message) override;
  bool AcceptWithResponder(mojo::Message* message,
      mojo::MessageReceiverWithStatus* responder) override;

 private:
  BackgroundSyncService* sink_;
  mojo::internal::SerializationContext serialization_context_;
  mojo::internal::ControlMessageHandler control_message_handler_;
};
class BackgroundSyncServiceClientStub : public mojo::MessageReceiverWithResponderStatus {
 public:
  BackgroundSyncServiceClientStub();
  ~BackgroundSyncServiceClientStub() override;
  void set_sink(BackgroundSyncServiceClient* sink) { sink_ = sink; }
  BackgroundSyncServiceClient* sink() { return sink_; }
  mojo::internal::SerializationContext* serialization_context() {
    return &serialization_context_;
  }

  bool Accept(mojo::Message* message) override;
  bool AcceptWithResponder(mojo::Message* message,
      mojo::MessageReceiverWithStatus* responder) override;

 private:
  BackgroundSyncServiceClient* sink_;
  mojo::internal::SerializationContext serialization_context_;
  mojo::internal::ControlMessageHandler control_message_handler_;
};
class BackgroundSyncServiceRequestValidator : public mojo::MessageFilter {
 public:
  explicit BackgroundSyncServiceRequestValidator(mojo::MessageReceiver* sink = nullptr);

  bool Accept(mojo::Message* message) override;
};
class BackgroundSyncServiceClientRequestValidator : public mojo::MessageFilter {
 public:
  explicit BackgroundSyncServiceClientRequestValidator(mojo::MessageReceiver* sink = nullptr);

  bool Accept(mojo::Message* message) override;
};
class BackgroundSyncServiceResponseValidator : public mojo::MessageFilter {
 public:
  explicit BackgroundSyncServiceResponseValidator(mojo::MessageReceiver* sink = nullptr);

  bool Accept(mojo::Message* message) override;
};
class BackgroundSyncServiceClientResponseValidator : public mojo::MessageFilter {
 public:
  explicit BackgroundSyncServiceClientResponseValidator(mojo::MessageReceiver* sink = nullptr);

  bool Accept(mojo::Message* message) override;
};



}  // namespace content

#endif  // CONTENT_COMMON_BACKGROUND_SYNC_SERVICE_MOJOM_H_